--[[
Author:        David Szybala
Website:       www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:       EXR_Channel_Loader.lua
Date:          5/9/18
Description:   This script will load all aux channels
               of an EXR sequence
]]


function get_file_path()
    --[[Creates a dialog asking the user for a file path]]
    local ui_controls = {}
    ui_controls[1] = {"path_text", Name = "Select An Image", "ClipBrowse"}
    local ui = composition:AskUser("EXR Channel Loader", ui_controls)
    return ui
end
    
    
function load_file_path(full_path, pos_x, pos_y)
    --[[Creates a loader with image "full_path" at position ["pos_x", "pos_y"]
    and returns the node]]
    local new_loader = composition:AddTool("Loader", pos_x, pos_y)
    new_loader.Clip[1] = full_path
    return new_loader
end


function disable_unused_channels(loader, disable_alpha)
    --[[Disables all unused channels for "loader" ]]
    if disable_alpha then
        loader.Clip1.OpenEXRFormat.AlphaEnable = false
    end
    loader.Clip1.OpenEXRFormat.ZEnable = false
    loader.Clip1.OpenEXRFormat.CovEnable = false
    loader.Clip1.OpenEXRFormat.ObjIDEnable = false
    loader.Clip1.OpenEXRFormat.MatIDEnable = false
    loader.Clip1.OpenEXRFormat.UEnable = false
    loader.Clip1.OpenEXRFormat.VEnable = false
    loader.Clip1.OpenEXRFormat.XNormEnable = false
    loader.Clip1.OpenEXRFormat.YNormEnable = false
    loader.Clip1.OpenEXRFormat.ZNormEnable = false
    loader.Clip1.OpenEXRFormat.XVelEnable = false
    loader.Clip1.OpenEXRFormat.YVelEnable = false
    loader.Clip1.OpenEXRFormat.ZVelEnable = false
    loader.Clip1.OpenEXRFormat.XRevVelEnable = false
    loader.Clip1.OpenEXRFormat.YRevVelEnable = false
    loader.Clip1.OpenEXRFormat.XPosEnable = false
    loader.Clip1.OpenEXRFormat.YPosEnable = false
    loader.Clip1.OpenEXRFormat.ZPosEnable = false
    loader.Clip1.OpenEXRFormat.XDispEnable = false
    loader.Clip1.OpenEXRFormat.YDispEnable = false
end


function is_table_value(search_value, search_table)
    --[[Returns true if value "search_value" is in table "search_table", false otherwise]]
    for k,v in pairs(search_table) do
        if search_value == v then
            return true
        end
    end
    
    return false
end


function is_table_key(search_key, search_table)
    --[[Returns true if key "search_value" is in table "search_table", false otherwise]]
    for k,v in pairs(search_table) do
        if search_key == k then
            return true
        end
    end
    
    return false
end


function get_aux_channels(loader)
    --[[Returns a dictionary of channels that need to be loaded. Format is
    <channel name>: [<red value>, <green value>, <blue value>] ]]
    local load_channels = {}
    local channels_to_sort = {}
    local ignore_channels = {}
    
    ignore_channels[1] = "SomethingThatWontMatchHopefully"
    ignore_channels[2] = "R"
    ignore_channels[3] = "G"
    ignore_channels[4] = "B"
    ignore_channels[5] = "A"
    
    --remove standard channels before processing
    for k,v in pairs(loader.Clip1.OpenEXRFormat.RedName:GetAttrs("INPIDT_ComboControl_ID")) do
        if not (is_table_value(v, ignore_channels)) then
            table.insert(channels_to_sort, v)
        end
    end
    
    --group channels by name and values
    for i,v in ipairs(channels_to_sort) do
        n = ""
        index = string.find(v, "%.")

        if index ~= nil then
            n = string.sub(v, 1, -3)
        end
        
        if v == "Z" then
            load_channels["Z_Depth"] = {"Z", "Z", "Z"}
        elseif string.lower(string.sub(v, -2, -1)) == ".a" then
            load_channels[n] = {v, v, v}
        else
            if is_table_key(n, load_channels) then
                if string.lower(string.sub(v, -2, -1)) == ".r" then
                    load_channels[n][1] = v
                elseif string.lower(string.sub(v, -2, -1)) == ".g" then
                    load_channels[n][2] = v
                elseif string.lower(string.sub(v, -2, -1)) == ".b" then
                    load_channels[n][3] = v
                elseif string.lower(string.sub(v, -2, -1)) == ".x" then
                    load_channels[n][1] = v
                elseif string.lower(string.sub(v, -2, -1)) == ".y" then
                    load_channels[n][2] = v
                elseif string.lower(string.sub(v, -2, -1)) == ".z" then
                    load_channels[n][3] = v
                end
            else
                load_channels[n] = {v, v, v}
            end
        end
    end

    return load_channels
end


function copy_loader_attrs(source_loader, dest_loader)
    --[[Copies the global in/out, trim in/out, etc of the 'source_loader' to
    the 'dest_loader' ]]
    dest_loader.GlobalOut = source_loader:GetAttrs("TOOLNT_Clip_End")[1]  --CRITICAL! You must set out before in!
    dest_loader.GlobalIn = source_loader:GetAttrs("TOOLNT_Clip_Start")[1]
    dest_loader.ClipTimeStart = source_loader:GetAttrs("TOOLIT_Clip_TrimIn")[1]
    dest_loader.ClipTimeEnd = source_loader:GetAttrs("TOOLIT_Clip_TrimOut")[1]
    dest_loader.HoldFirstFrame = source_loader:GetAttrs("TOOLIT_Clip_ExtendFirst")[1]
    dest_loader.HoldLastFrame = source_loader:GetAttrs("TOOLIT_Clip_ExtendLast")[1]
end


function load_image_channels(loader)
    --[[Loads all aux channels from "loader" ]]
    local pos = composition.CurrentFrame.FlowView:GetPosTable(loader)
    local file_path = loader:GetAttrs("TOOLST_Clip_Name")[1.0]
    local channels = get_aux_channels(loader)
    local offset = 1
    
    --if force tiles then adjust spacing offset
    if (fusion:GetPrefs("Comp.FlowView.ForceAll") or fusion:GetPrefs("Comp.FlowView.ForceSource")) then
        offset = 3
        pos[2] = pos[2] + 4
    else
        pos[2] = pos[2] + 2
    end

    --load the aux channels and copy attrs
    for k,v in pairs(channels) do
        new_loader = load_file_path(file_path, pos[1], pos[2])
        new_loader:SetAttrs({TOOLS_Name = (new_loader:GetAttrs("TOOLS_Name") .. "_" .. (string.gsub(k, " ", "_")))})
        new_loader.Clip1.OpenEXRFormat.RedName = v[1]
        new_loader.Clip1.OpenEXRFormat.GreenName = v[2]
        new_loader.Clip1.OpenEXRFormat.BlueName = v[3]
        disable_unused_channels(new_loader, true)
        copy_loader_attrs(loader, new_loader)
        pos[2] = pos[2] + offset
    end
end


function main()
    --[[Main entry point.  Loads in all channels of selected loaders with
    exr sequences. If no loaders are selected, the user can select a new
    exr ]]
    local selected_loaders = composition:GetToolList(true, "Loader")
    local ui_results = nil
    
    if next(selected_loaders) == nil then
        ui_results = get_file_path()
        
        if ui_results == nil then
            print("WARNING: User cancelled dialog")
            return false
            
        elseif ui_results["path_text"] == "" then
            print("ERROR: No path entered")
            return false
            
        else
            selected_loaders = {}
            selected_loaders[1] = (load_file_path(ui_results["path_text"]))
            disable_unused_channels(selected_loaders[1])
            composition.CurrentFrame.FlowView:Select(selected_loaders[1], true)
        end
    end
    
    for k,v in pairs(selected_loaders) do
        if v:GetAttrs("TOOLST_Clip_Name") ~= "" then
            if v:GetAttrs("TOOLST_Clip_FormatName")[1.0] == "OpenEXRFormat" then
                load_image_channels(v)
            end
        end
    end
    
    print("INFO: EXR channel loader complete")
end


composition:Lock()
main()
composition:Unlock()